//////////////////////////////////////////////////////////////////////////////////
// Module	 : sx16core.h														//
// Function  : Include for Sensoray 516/616 application and driver projects.	//
// Author    : Jim Lamberson, Charlie Liu	     								//
// Revision :											                        //
//             Sept, 2004   Phillip W.		 Made lin2.6 and Windows compatible //
//			   May, 2004    Alexander K.     Added RTP and some playback        //
//											 sync'ing stuff                     //
//             Sept, 2003   Jim Lamberson    release sx16-0.7					//
//             Oct., 2003   Charlie X. Liu   Added caption-related definitions	//
// Copyright : (C) Sensoray 2003												//
//////////////////////////////////////////////////////////////////////////////////

#ifndef DEF_SX16CORE_H
#define DEF_SX16CORE_H

// ioctl constants ----------------------------------------------------

// Video system.
#define SYS_PAL						1			// PAL.
#define SYS_NTSC					2			// NTSC.

// Video input type.
#define CVIDEO						0			// Composite video.
#define SVIDEO						1			// S-Video.

// Video input status bits.
#define VIDEO_DET					1			// Video signal detected.
#define F60HZ_DET					0x20		// Field frequency 60 Hz detected.

// Picture sizes/stream format.
#define SIZE176_1					4			// QSIF,     MPEG-1, NTSC-480x480 PAL-480x576
#define SIZE352_1					3			// SIF,      MPEG-1, NTSC-352x240 PAL-352x288
#define SIZE352_2					2			// 1/2 D1,   MPEG-2, NTSC-352x480 PAL-352x576
#define SIZE480_2					1			// 2/3 D1,   MPEG-2, NTSC-480x480 PAL-480x576 SVCD mux(standard)
#define SIZE640_2					6			// D1 H:640, MPEG-2, NTSC-352x240 PAL-352x288
#define SIZE704_2					0			// D1 H:704, MPEG-2, NTSC-704x480 PAL-704x576
#define SIZE720_2					5			// D1 H:720, MPEG-2, NTSC-352x240 PAL-352x288
#define SIZE352_1_2					7			// SIF,      MPEG-1, NTSC-352x240 PAL-352x288, MPEG1 stream with MPEG2 mux
#define SIZE480_2_2					8			// 2/3 D1,   MPEG-2, NTSC-480x480 PAL-480x576, DVD MUX(for playback)

// Stream type.
#define STREAMTYPE_MUX				0			// Multiplexed program/system stream.
#define STREAMTYPE_VES				1			// Video elementary stream.
#define STREAMTYPE_AES				2			// Audio elementary stream.
#define STREAMTYPE_TRANSPORT		3			// Transport stream

// Audio encoding format.
#define AFMT_MPEG1_LAYER1			0			// MPEG-1 layer 1.
#define AFMT_MPEG1_LAYER2			1			// MPEG-1 layer 2.
#define AFMT_MPEG1_LAYER3			2			// MPEG-1 layer 3.
#define AFMT_AC3					3			// AC3 2-channel.

// Stereo mode for MPEG audio.
#define STEREOMODE_STEREO			0			// Stereo.
#define STEREOMODE_JOINT			1			// Joint stereo (intensity or ms).
#define STEREOMODE_DUAL				2			// Dual channel.
#define STEREOMODE_SINGLE			3			// Single channel.

// Audio sampling rate.
#define AUDIO_SAMPLERATE_44KHZ		0			// 44.1 KHz.
#define AUDIO_SAMPLERATE_48KHZ		1			// 48 KHz.
#define AUDIO_SAMPLERATE_32KHZ		2			// 32 KHz.

// Audio PCM resolution.
#define AUDIO_PCM_RES_16			0			// 16 bits.
#define AUDIO_PCM_RES_18			1			// 18 bits.
#define AUDIO_PCM_RES_20			2			// 20 bits.
#define AUDIO_PCM_RES_24			3			// 24 bits.

// Multiplexing audio/video select.  Not applicable to elementary streams.
#define AVSEL_BOTH					0			// Audio and video.
#define AVSEL_VIDEO_ONLY			1			// Video only.
#define AVSEL_AUDIO_ONLY			2			// Audio only.

// System format.
#define SYSFMT_SVCD					0			// Super VCD.
#define SYSFMT_VCD					1			// VCD.
#define SYSFMT_OTHER				2			// Other formats.

// Multiplexing format.
#define MUX_FORMAT_SVCD				0			// Super VCD.
#define MUX_FORMAT_VCD				1			// VCD.
#define MUX_FORMAT_DVD				2			// DVD.
#define MUX_FORMAT_CDDA				3			// CDDA.
#define MUX_FORMAT_VES				4			// Video elementary stream.
#define MUX_FORMAT_AES				5			// Audio elementary stream.
#define MUX_FORMAT_TRANSPORT		6			// Transport stream.

// Caption position.
#define MIN_XSTART					0			// Minimum X start
#define MIN_YSTART					0			// Minimum Y start
#define MAX_XSTART					700			// Maximum X start
#define MAX_YSTART					478			// Maximum Y start
#define MAX_YSTART_NTSC				478			// Maximum Y start for NTSC
#define MAX_YSTART_PAL				574			// Maximum Y start for PAL
#define MAX_XSTOP					704			// Maximum X stop
#define MAX_YSTOP					478			// Maximum Y stop
#define MAX_YSTOP_NTSC				480			// Maximum Y stop for NTSC
#define MAX_YSTOP_PAL				576			// Maximum Y stop for PAL

// Caption overlay color.
#define OVL_WHITE					0			// white.
#define OVL_BLACK					1			// black.
#define OVL_RED						2			// red.
#define OVL_GREEN					3			// green.
#define OVL_BLUE					4			// blue.
#define OVL_YELLOW					5			// yellow.
#define OVL_CYAN					6			// cyan.
#define OVL_MAGENTA					7			// magenta.

// Caption overlay scale.
#define OVL_SCALE_X1				0			// scale = x1.
#define OVL_SCALE_X2				1			// scale = x2.
#define OVL_SCALE_X2V				2			// scale = x2 (only vertical).
#define OVL_SCALE_X4				3			// scale = x4.

// Caption transparency attribute.
#define OVL_TRANSP_T				1			// transparent.
#define OVL_TRANSP_BBKGND			2			// black background.
#define OVL_TRANSP_T_BKGND			3			// transparent text on a background.
#define OVL_TRANSP_B_BKGRD			4			// black text on a background.

// Caption on/off.
#define CAPTION_OFF					0			// caption off.
#define CAPTION_ON					1			// caption on.

// Error codes.
#define ERR_I2C 					0x0001		// I2C bus error.
#define ERR_UCODE_NOTFOUND			0x0002		// Microcode can't be opened.
#define ERR_UCODE_NOUPLOAD			0x0004		// Mpeg codec not acknowledging microcode upload.
#define ERR_MEM_ALLOC				0x0008		// Memory allocation error.
#define ERR_DEBI_TIMEOUT			0x0010		// DEBI transaction timed out (616 only).

// Mpeg codec microcode selectors.
#define MPEG_RECORD					0			// Convert stream to MPEG.
#define MPEG_PLAY					1			// Convert MPEG to stream.

// Attribute identifiers for SX16CORE_attr_get() and SX16CORE_attr_set().
#define ATTR_audio_bass				0			// Audio audio_bass level.
#define ATTR_audio_bitrate			1			// Audio bit rate.
#define ATTR_audio_format			2			// Audio format.
#define ATTR_audio_mute				3			// Audio muting.
#define ATTR_audio_pcm_resolution	4			// Audio PCM resolution.
#define ATTR_audio_sample_rate		5			// Audio sample rate.
#define ATTR_audio_stereo_mode		6			// Audio stereo mode.
#define ATTR_audio_treble			7			// Audio treble level.
#define ATTR_audio_volume			8			// Audio volume level.
// see more audio options below( added later for backward compatibility with existing drivers)
#define ATTR_av_select				9			// Audio/video select.
#define ATTR_video_bitrate			10			// Video bit rate in bps.
#define ATTR_video_brightness		11			// Video brightness level.
#define ATTR_video_chan				12			// Video input channel number.
#define ATTR_video_colorbars_enable	13			// Video color bar generation enable.
#define ATTR_video_contrast			14			// Video contrast level.
#define ATTR_video_filter_enable	15			// Video filter enable.
#define ATTR_video_m				16			// GOP length.
#define ATTR_video_n				17			// I-P distance.
#define ATTR_video_picsize			18			// Picture size and compression type (fundamental).
#define ATTR_video_saturation		19			// Video video_saturation.
#define ATTR_video_type				20			// Video input type: svideo or composite.
#define ATTR_video_system			21			// Video system.
#define ATTR_video_system_format	22			// Video system format (fundamental).
#define ATTR_video_vbr_enable		23			// Variable bit rate enable.
#define ATTR_mux_format				24			// Stream multiplex format (fundamental).
#define ATTR_program_number			25			// Program number.
#define ATTR_mpeg					26			// Picture size and compression type (derived).
#define ATTR_stream_type			27			// Stream multiplex type (derived).
// microphone audio input settings.  Leave as default for line-in
#define ATTR_audio_inampgain        28          // audio input channel amplifier 0x00(-3db) to 0x7f (+60.5dB) in steps of 0.5db
#define ATTR_audio_agc_onoff        29          // microphone AGC.  0--off, 1--on.  If on, ATTR_mic_inampgain is ignored.
#define ATTR_audio_agc_sens         30          // microphone sensitivity for AGC only. 0=-3db, 1=0db, 2=+3db, 3=+9, 4=+15, 5=+21, 6=+27db



#define NUM_ATTR_IDS				31			// Number of legal attribute identifiers.

// Attribute identifiers for SX16CORE_caption_get() and SX16CORE_caption_set().
#define CAPTN_window				0			// Caption window #.
#define CAPTN_xstart				1			// Caption x start.
#define CAPTN_xstop					2			// Caption x stop.
#define CAPTN_ystart				3			// Caption y start.
#define CAPTN_ystop					4			// Caption y stop.
#define CAPTN_scale					5			// Caption scale.
#define CAPTN_color					6			// Caption color.
#define CAPTN_transp				7			// Caption transparency.
#define CAPTN_text					8			// Caption text.
#define CAPTN_on					9			// Caption on/off.

#define NUM_CAPTN_IDS			   10			// Number of legal attribute identifiers.

// Queue selection constants.
#define Q_ID_AUD					1			// Audio queue.
#define Q_ID_MUX					2			// Multiplexed queue.
#define Q_ID_VID					3			// Video queue.

// capture settings
#define NUM_CAP_INPUTS  2                                                                               
#define NUM_CAP_OUTPUTS 2

///////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////// STRUCTURAL CONFIGURATION CONSTANTS  ////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////

// SM dma fifo transfer size.  Theoretically, this may be any power of 2 in the range 8 to 32768.  The SM documentation suggests
// that this be specified as an integral multiple of 256.  In practice, however, 31K is the only value that seems to work reliably
// for both the Play and Record modes.
#define DFLT_DMA_SIZE		( 1024 * 31 )	// SM's mpeg dma transfer size.

// Block transfer size, in bytes, between the SM and our private mpeg buffer.  Larger sizes work in playback mode,
// but will cause problems in record mode.
#define CHUNK_SIZE			( 1024 )		// SM's block transfer size in bytes.
#define CHUNK_SIZE_PLAY		( 1024 )// SM's block transfer size in bytes.
// for playback


// Types --------------------------------------------------------------------
typedef void * HBD;	// Logical handle to X16 board.


#ifndef WIN32
typedef unsigned long long int __uint64;
#else
typedef unsigned __int64 __uint64;
#endif


typedef struct {
    unsigned short 		SequenceNumber;             // SequenceNumber
    int 				LostPackets;	
    unsigned long 		PacketCount;	
    unsigned long       dblBufferCount;
} write_param_t;  // sensoray write params


// OS-independent functions implemented in sx16core.o -----------------------

#ifdef __cplusplus
extern "C"
{
#endif
HBD 				SX16CORE_board_init			( unsigned long product_id, unsigned long base_adrs, unsigned long irq, void *dma_adrs_virt, unsigned long dma_adrs_phys );
HBD 				SX16CORE_board_init_debug	( unsigned long product_id, unsigned long base_adrs, unsigned long irq, void *dma_adrs_virt, unsigned long dma_adrs_phys, unsigned long debug_flags );
unsigned long		SX16CORE_attr_get			( HBD board, unsigned long attr_id );
void				SX16CORE_attr_set			( HBD board, unsigned long attr_id, unsigned long value );
unsigned long		SX16CORE_caption_get		( HBD board, unsigned long cap_attr_id );
void				SX16CORE_caption_set		( HBD board, unsigned long cap_attr_id, unsigned long value );

int 				SX16CORE_in_status_get 		( HBD board );

int 				SX16CORE_board_close 		( HBD board );
int 				SX16CORE_board_open 		( HBD board );
void 				SX16CORE_board_reset 		( HBD board );
void 				SX16CORE_board_shutdown		( HBD board );
int					SX16CORE_board_is_516		( unsigned short base_adrs );
unsigned long		SX16CORE_get_stat_KB		( HBD board );
int					SX16CORE_gpio_read			( HBD board );
void 				SX16CORE_gpio_write 		( HBD board, int state );
unsigned short		SX16CORE_microcode_version	( HBD board );
int					SX16CORE_is_dma_set			( HBD board );
unsigned long		SX16CORE_get_dma_remaining	( HBD board );
int					SX16CORE_poll_dma_flag		( HBD board );
int					SX16CORE_reset_interrupt	( HBD board );
unsigned long		SX16CORE_overlay_get_height ( HBD board );
unsigned long		SX16CORE_overlay_get_width	( HBD board );
void				SX16CORE_overlay_set_c1 	( HBD board, unsigned long val );
void				SX16CORE_overlay_set_c2 	( HBD board, unsigned long val );
void				SX16CORE_overlay_set_column ( HBD board, unsigned long val );
void				SX16CORE_overlay_set_data	( HBD board, unsigned long val );
void				SX16CORE_overlay_set_enable	( HBD board, int enable );
void				SX16CORE_overlay_set_row	( HBD board, unsigned long val );
void				SX16CORE_overlay_set_xend	( HBD board, unsigned long val );
void				SX16CORE_overlay_set_xstart ( HBD board, unsigned long val );
void				SX16CORE_overlay_set_yend	( HBD board, unsigned long val );
void				SX16CORE_overlay_set_yfirst ( HBD board, unsigned long val );
void				SX16CORE_overlay_set_ynext	( HBD board, unsigned long val );
void				SX16CORE_overlay_set_ystart ( HBD board, unsigned long val );
unsigned long		SX16CORE_queue_get_any_size	( HBD board, int qid );
void				SX16CORE_start_decode		( HBD board );
void				SX16CORE_start_encode		( HBD board );
void				SX16CORE_stream_bypass		( HBD board, int enable );
int					SX16CORE_stream_is_decoding	( HBD board );
int					SX16CORE_stream_is_encoding	( HBD board );
void				SX16CORE_stream_pause		( HBD board );
int					SX16CORE_stream_read		( HBD board, unsigned char *buffer, unsigned long nbytes );
void				SX16CORE_stream_resume 		( HBD board );
void				SX16CORE_stream_stop		( HBD board );
int					SX16CORE_stream_write		( HBD board, const unsigned char *buffer, unsigned long nbytes );
int 				SX16CORE_get_frame_rate 	( HBD board );
void *				SX16CORE_get_user_data		( HBD board );
__uint64			SX16CORE_get_elapsed_time	( HBD board );
int 				SX16CORE_acquire_mutex		( HBD board );
void 				SX16CORE_release_mutex		( HBD board );
#ifndef WIN32
int                 SX16CORE_read_rtp           ( HBD board, unsigned char *buffer );
int                 SX16CORE_rtp_plus           ( HBD board, unsigned char *buffer);
int                 SX16CORE_get_rtp_handle     ( HBD board);
int 				ReadStreamRTPParse			( HBD board, int RTP_handle );


// capture functions
int                 SX16CORE_capture_start      ( HBD board ); // 616
int                 SX16CORE_capture_get_status ( HBD board );
int                 SX16CORE_capture_close      ( HBD board ); // (616)
int                 SX16CORE_capture_init       ( HBD board );

    
int                 SX16CORE_capture_set_fmt    ( HBD board, void *pFmt);  // vformat IN pointer
int                 SX16CORE_capture_get_fmt    ( HBD board , void *pfmt);
int                 SX16CORE_capture_set_capbuffer( HBD board, void *val);

//  sensoray write functions
unsigned long       SX16CORE_write_sensoray (HBD board, char *buffer, int skip_data, int reset_step );
unsigned long       SX16CORE_write_sensoray_mux (HBD board, char *buffer, int skip_data, int reset_step );
int                 SX16CORE_get_write_params( HBD board, write_param_t *pWriteParams); 
#endif

#ifdef __cplusplus
}
#endif


#endif		// #ifndef DEF_SX16CORE_H

